install.packages(c("acs", "spdep", "RColorBrewer", "car", "lmtest"), dependencies=T)

library(acs)
library(spdep)
library(RColorBrewer)
library(car)
library(lmtest)

#I had to look at the csv file to find which columns were which, since factfinder doens't download table numbers

social<-read.acs("C:/Users/ozd504/Dropbox/spatialDemography/column2data/ACS_09_5YR_DP5YR2_with_ann.csv",  geocols=3:1, skip=0)
economic<-read.acs("C:/Users/ozd504/Dropbox/spatialDemography/column2data/ACS_09_5YR_DP5YR3_with_ann.csv",  geocols=3:1, skip=0)

popsize<-social@estimate[,171] 
fertrate<-social@estimate[,77]
pnohs<-social@estimate[,118]+social@estimate[,120]
pfornborn<-social@estimate[,184]
pspanspkhh<-social@estimate[,228]
pmarriedwomen<-social@estimate[,64]
popsize<-social@estimate[,171]
tract<-social@geography$GEO.id2

unemp<-economic@estimate[,17]
medhhinc<-economic@estimate[,125]/1000
poverty<-economic@estimate[,205]


dat<-data.frame(popsize=popsize, fertrate=fertrate, pnohs=pnohs, pfornborn=pfornborn, pspanspkhh=pspanspkhh, pmarriedwomen=pmarriedwomen, unemp=unemp, medhhinc=medhhinc, poverty=poverty, tract=tract)
head(dat)



geodat<-readShapePoly("C:/Users/ozd504/Dropbox/spatialDemography//48_TEXAS/48029_Bexar_County/tl_2009_48029_tract00.shp", proj4string=CRS('+proj=longlat +datum=NAD83'))

mdat<-geodat@data #temp file
mdat<-merge(x=mdat, y=dat, by.x="CTIDFP00", by.y="tract", all.x=T, sort=F) #temp file
geodat@data<-mdat
rm(mdat) #remove temp file
keep<-!is.na(geodat$fertrate)
geodat<-geodat[keep,]

jpeg(filename="C:/Users/ozd504/Dropbox/spatialDemography/Col2nohs.jpg", quality=100)
spplot(geodat, "pnohs",
       at=quantile(geodat$pnohs, p=c(0,.25, .5, .75, 1), na.rm=T),
       col.regions=brewer.pal(5, "Reds"), main="Cloropleth map of Bexar County", sub="% Without High School Education")
dev.off()

hist(geodat$pnohs)
hist(log(geodat$pnohs))

fit.1<-lm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc), data=geodat)
summary(fit.1)


jpeg(filename="C:/Users/ozd504/Dropbox/spatialDemography/Col2Mod1.jpg", quality=100)
par(mfrow=c(3,2))
hist(dat$pnohs, main="Distribution of Outcome")
hist(rstudent(fit.1), main="Model Residuals")
plot(fit.1)
dev.off()


#variance inflation factor for beta's
vif(fit.1)
#Breush-Pagan test for constant variance
bptest(fit.1) 
#looks like we have heteroskedasticity


fit.1wt<-lm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc),weights=popsize, data=geodat)
summary(fit.1wt)

bptest(fit.1wt)

Anova(fit.1, white.adjust=T)

#extract studentized residuals from the fit, and examine them
geodat$residfit1<-rstudent(fit.1)
cols<-brewer.pal(7,"RdBu")

jpeg(filename="C:/Users/ozd504/Dropbox/spatialDemography/Col2Res1.jpg", quality=100)
spplot(geodat,"residfit1", at=quantile(geodat$residfit1), col.regions=cols, main="Residuals from OLS Fit")
dev.off()

#Make spatial weights
sa.nb<-poly2nb(geodat, queen=T)
sa.wt<-nb2listw(sa.nb, style="W")

#test for residual autocorrelation 
lm.morantest(fit.1, listw=sa.wt)
#looks like we have significant autocorrelation in our residuals

#Let's look at the local autocorrelation in our residuals
#get the values of I
geodat$lmfit1<-localmoran(geodat$residfit1, sa.wt)[,"Z.Ii"]

jpeg(filename="C:/Users/ozd504/Dropbox/spatialDemography/Col2LocalM.jpg", quality=100)
spplot(geodat, "lmfit1", at=quantile(geodat$lmfit1)
       , col.regions=brewer.pal(6, "RdBu"),main="Local Moran's I of OLS Residuals")
dev.off()

#perform a model specification test
lm.LMtests(fit.1, listw=sa.wt, test="all")

#Now we fit the spatial lag model 
#The lag mode is fit with lagsarlm() in the spdep library
#we basically specify the same model as in the lm() fit above
#But we need to specify the spatial weight matrix and the type
#of lag model to fit

fit.lag<-lagsarlm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc), data=geodat, listw=sa.wt, type="lag")
summary(fit.lag, Nagelkerke=T)

#which says we still have significant autocorrelation in the residuals, even after
#accounting for autocorrelation in the outcome
bptest.sarlm(fit.lag)



#next we fit the spatial error model
fit.err<-errorsarlm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc), data=geodat, listw=sa.wt)

summary(fit.err, Nagelkerke=T)

#Next we fit the spatial durbin model, with lagged predictors
fit.lag.durbin<-lagsarlm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc), data=geodat, listw=sa.wt, type="mixed")

summary(fit.lag.durbin, Nagelkerke=T)
bptest.sarlm(fit.lag.durbin)


#Simultaneous autoregressive model
fit.sac<-sacsarlm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc), data=geodat, listw=sa.wt, type="sac")

summary(fit.sac, Nagelkerke=T)

fit.sac.mix<-sacsarlm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc), data=geodat, listw=sa.wt, type="sacmixed")

summary(fit.sac.mix, Nagelkerke=T)

bptest.sarlm(fit.sac)


#AIC for three spatial models and OLS model
AIC(fit.1)
AIC(fit.lag)
AIC(fit.err)
AIC(fit.lag.durbin)
AIC(fit.sac)
AIC(fit.sac.mix)

#impacts for lag models
im.sdm<-impacts(fit.lag.durbin,listw=sa.wt, R=100)
summary( im.sdm, zstats=TRUE)

im.lag<-impacts(fit.lag, listw=sa.wt, R=100)
summary(im.lag, zstats=T)


#spatial filtering
sp.filt<-SpatialFiltering(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc), data=geodat,nb=sa.nb)
sp.filt

fit.filt<-lm(pnohs~pfornborn+pmarriedwomen+pspanspkhh+unemp+poverty+log(medhhinc)+fitted(sp.filt), data=geodat)
summary(fit.filt)
AIC(fit.filt)
Anova(fit.filt, white.adjust=T)